@@ -8,6 +8,7 @@ from .flowlayout import FlowLayout
 from .colorchooserbutton import ColorChooserButton
 from .filechooserbutton import FileChooserButton
 from .fontchooserbutton import FontChooserButton
+from .inputandslider import InputAndSlider
 from .spanslider import SpanSlider
 from .labeledslider import LabeledSlider
 from .waitingspinner import WaitingSpinner
@@ -23,6 +24,7 @@ __all__ = ["FlowLayout",
            "ColorChooserButton",
            "FileChooserButton",
            "FontChooserButton",
+           "InputAndSlider",
            "SpanSlider",
            "LabeledSlider",
            "WaitingSpinner",
