@@ -3,9 +3,17 @@
 @author: Philipp Temminghoff
 """
 
-from qtpy import QtWidgets, QtCore, QtGui
-
 import qtawesome as qta
+from qtpy import QtCore, QtGui, QtWidgets
+
+from prettyqt import core, gui, widgets
+from prettyqt.utils import bidict
+
+TAB_SHAPES = bidict(rounded=QtWidgets.QTabWidget.Rounded,
+                    triangular=QtWidgets.QTabWidget.Triangular)
+
+
+QtWidgets.QTabWidget.__bases__ = (widgets.Widget,)
 
 
 class TabWidget(QtWidgets.QTabWidget):
@@ -13,33 +21,97 @@ class TabWidget(QtWidgets.QTabWidget):
     Widget for managing the tabs section
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, parent=None, closable=False, detachable=False):
 
         # Basic initalization
-        super().__init__(*args, **kwargs)
-        self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
+        super().__init__(parent)
         self.tabCloseRequested.connect(self.remove_tab)
-        self.tab_bar = TabBar(self)
-        self.tab_bar.on_detach.connect(self.detach_tab)
+        self.tab_bar = widgets.TabBar(self)
 
         self.setTabBar(self.tab_bar)
 
         # Used to keep a reference to detached tabs since their QMainWindow
         # does not have a parent
         self.detached_tabs = dict()
+        if detachable:
+            self.set_detachable()
+        self.set_closable(closable)
 
-        # Close all detached tabs if the application is closed explicitly
-        QtWidgets.qApp.aboutToQuit.connect(self.close_detached_tabs)
+    def __len__(self):
+        return self.count()
 
-        self.setEnabled(True)
-        font = QtGui.QFont()
-        font.setPointSize(9)
-        self.setFont(font)
-        self.setTabsClosable(True)
+    def __getitem__(self, index):
+        if isinstance(index, int):
+            return self.widget(index)
+        else:
+            return self.findChild(QtWidgets.QWidget, index)
+
+    def __getstate__(self):
+        return dict(tabbar=self.tabBar(),
+                    widgets=self.get_children(),
+                    movable=self.isMovable(),
+                    document_mode=self.documentMode(),
+                    current_index=self.currentIndex(),
+                    tab_shape=self.get_tab_shape(),
+                    # elide_mode=self.get_elide_mode(),
+                    icon_size=self.iconSize())
+
+    def __setstate__(self, state):
+        self.__init__()
+        self.setTabBar(state["tabbar"])
+        self.setDocumentMode(state.get("document_mode", False))
+        self.setMovable(state.get("movable", False))
+        self.set_tab_shape(state.get("tab_shape", "rounded"))
+        self.setIconSize(state["icon_size"])
+        for (widget, name, icon, tooltip, whatsthis) in state["widgets"]:
+            i = self.add_tab(widget, name, icon)
+            self.setTabToolTip(i, tooltip)
+            self.setTabWhatsThis(i, whatsthis)
+        self.setCurrentIndex(state.get("index", 0))
+
+    def set_tab_shape(self, shape: str):
+        """set tab shape for the tabwidget
+
+        Valid values are "rounded" and "triangular"
+
+        Args:
+            shape: tab shape to use
+
+        Raises:
+            ValueError: tab shape does not exist
+        """
+        if shape not in TAB_SHAPES:
+            raise ValueError("Invalid value for shape.")
+        self.setTabShape(TAB_SHAPES[shape])
+
+    def get_tab_shape(self) -> str:
+        """returns tab shape
+
+        possible values are "roundes", "triangular"
+
+        Returns:
+            tab shape
+        """
+        return TAB_SHAPES.inv[self.tabShape()]
+
+    def get_children(self):
+        return [(self.widget(i), self.tabText(i), self.tab_icon(i),
+                 self.tabToolTip(i), self.tabWhatsThis(i))
+                for i in range(self.count())]
+
+    def tab_icon(self, i):
+        return gui.Icon(self.tabIcon(i))
+
+    def set_detachable(self):
+        self.tab_bar.on_detach.connect(self.detach_tab)
+        widgets.app().aboutToQuit.connect(self.close_detached_tabs)
         self.setMovable(True)
 
-    @QtCore.Slot(int, QtCore.QPoint)
-    def detach_tab(self, index, point):
+    def set_closable(self, closable: bool = True):
+        self.setTabsClosable(closable)
+
+    @core.Slot(int, QtCore.QPoint)
+    def detach_tab(self, index: int, point: QtCore.QPoint):
         """
         Detach the tab by removing it's contents and placing them in
         a DetachedTab window
@@ -61,8 +133,8 @@ class TabWidget(QtWidgets.QTabWidget):
 
         # Create a new detached tab window
         detached_tab = DetachedTab(name, widget)
-        detached_tab.setWindowModality(QtCore.Qt.NonModal)
-        detached_tab.setWindowIcon(icon)
+        detached_tab.set_modality("none")
+        detached_tab.set_icon(icon)
         detached_tab.setGeometry(widget_rect)
         detached_tab.on_close.connect(self.attach_tab)
         detached_tab.move(point)
@@ -71,8 +143,13 @@ class TabWidget(QtWidgets.QTabWidget):
         # Create a reference to maintain access to the detached tab
         self.detached_tabs[name] = detached_tab
 
-    def add_tab(self, widget, label, icon=None):
-        if icon is None:
+    def add_tab(self, item, label, icon=None):
+        if isinstance(item, QtWidgets.QLayout):
+            widget = widgets.Widget()
+            widget.set_layout(item)
+        else:
+            widget = item
+        if not icon:
             return self.addTab(widget, label)
         else:
             if isinstance(icon, str):
@@ -80,7 +157,7 @@ class TabWidget(QtWidgets.QTabWidget):
             return self.addTab(widget, icon, label)
 
     def insert_tab(self, pos, widget, label, icon=None):
-        if icon is None:
+        if not icon:
             return self.insertTab(pos, widget, label)
         else:
             if isinstance(icon, str):
@@ -113,18 +190,19 @@ class TabWidget(QtWidgets.QTabWidget):
         self.setCurrentIndex(index)
 
     def close_detached_tabs(self):
-        #  Close all tabs that are currently detached.
+        """Close all tabs that are currently detached
+        """
         for detached_tab in self.detached_tabs.values():
             detached_tab.close()
 
-    @QtCore.Slot(int)
+    @core.Slot(int)
     def remove_tab(self, index: int):
         widget = self.widget(index)
         self.removeTab(index)
         if widget is not None:
             widget.deleteLater()
 
-    @QtCore.Slot(object, str)
+    @core.Slot(object, str)
     def open_widget(self, widget: QtWidgets.QWidget, title: str = "Unnamed"):
         """
         create a tab containing delivered widget
@@ -132,34 +210,26 @@ class TabWidget(QtWidgets.QTabWidget):
         index = self.add_tab(widget, title, icon="mdi.widgets")
         self.setCurrentIndex(index)
 
-
-class TabBar(QtWidgets.QTabBar):
-    on_detach = QtCore.Signal(int, QtCore.QPoint)
-
-    def __init__(self, parent=None):
-        super().__init__(parent)
-
-        self.setAcceptDrops(True)
-        self.setElideMode(QtCore.Qt.ElideRight)
-        self.setSelectionBehaviorOnRemove(QtWidgets.QTabBar.SelectLeftTab)
-        self.mouse_cursor = QtGui.QCursor()
-
-    #  Send the on_detach when a tab is double clicked
-    def mouseDoubleClickEvent(self, event):
-        event.accept()
-        self.on_detach.emit(self.tabAt(event.pos()), self.mouse_cursor.pos())
+    def set_tab(self, index, position: str, widget=None):
+        self.tabBar().set_tab(index, position, widget)
 
 
-#  When a tab is detached, the contents are placed into this QMainWindow.
-#  The tab can be re-attached by closing the dialog
-class DetachedTab(QtWidgets.QMainWindow):
-    on_close = QtCore.Signal(QtWidgets.QWidget, str, QtGui.QIcon)
+class DetachedTab(widgets.MainWindow):
+    """window containing a detached tab
+
+    When a tab is detached, the contents are placed into this QMainWindow.
+    The tab can be re-attached by closing the dialog
+
+    Attributes:
+        on_close: signal, emitted when window is closed (widget, title, icon)
+    """
+    on_close = core.Signal(QtWidgets.QWidget, str, QtGui.QIcon)
 
     def __init__(self, name, widget):
         super().__init__(None)
 
-        self.setObjectName(name)
-        self.setWindowTitle(name)
+        self.id = name
+        self.title = name
 
         self.widget = widget
         self.setCentralWidget(self.widget)
@@ -168,14 +238,15 @@ class DetachedTab(QtWidgets.QMainWindow):
     #  If the window is closed, emit the on_close and give the
     #  content widget back to the DetachableTabWidget
     def closeEvent(self, event):
-        self.on_close.emit(self.widget, self.objectName(), self.windowIcon())
+        self.on_close.emit(self.widget, self.id, self.windowIcon())
 
 
 if __name__ == "__main__":
-    import sys
-    app = QtWidgets.QApplication(sys.argv)
+    app = widgets.app()
     tab_widget = TabWidget()
-    widget = QtWidgets.QWidget()
+    widget = widgets.Widget()
     tab_widget.add_tab(widget, "Test")
+    widget_2 = widgets.Widget()
+    tab_widget.add_tab(widget_2, "Test 2")
     tab_widget.show()
     app.exec_()
